#include "pic32_c.h"
#include "init.h"

static const char message[] = "Hello!\r\n";

static void uart_write(char c)
{
    while (REG(UART_REG(1, UxSTA)) & (1 << 9)); /* UTXBF (buffer full) */

    REG(UART_REG(1, UxTXREG)) = c;
}

static void bits(uint32_t reg)
{
    uint32_t mask;

    for (mask = (1 << 31); mask; mask >>= 1)
        if (REG(reg) & mask)
            uart_write('1');
        else
            uart_write('0');

    uart_write('\r');
    uart_write('\n');
}

static void blink(uint32_t delay, uint32_t port, uint32_t pins)
{
    uint32_t counter;

    /* Clear outputs (LED). */

    CLR_REG(port, pins);

    while (1)
    {
        counter = delay;

        while (counter--) __asm__("");	/* retain loop */

        /* Invert outputs (LED). */

        INV_REG(port, pins);
        bits(DMA_REG(0, DCHxCON));
        bits(DMA_REG(0, DCHxECON));
        bits(DMA_REG(0, DCHxINT));

        /* SET_REG(DMA_REG(0, DCHxECON), 1 << 7); */
    }
}

void main(void)
{
    init_memory();
    init_pins();
    init_outputs();

    unlock_config();
    config_uart();
    lock_config();

    /* Initiate DMA on UART receive interrupt. */

    init_dma();
    dma_init(0, 0, 3);
    dma_set_interrupt(0, U1RX, 1);
    dma_set_transfer(0, PHYSICAL(message), sizeof(message) - 1,
                        HW_PHYSICAL(UART_REG(1, UxTXREG)), 1,
                        1);
    dma_on(0);

    /* Set UART interrupt priority below CPU priority. */

    uart_init(1, 115200);
    uart_init_interrupt(1, 1, 3);
    uart_on(1);

    interrupts_on();

    blink(3 << 24, PORTA, 1 << 3);
}

void exception_handler(void)
{
    blink(3 << 12, PORTA, 1 << 3);
}

void interrupt_handler(void)
{
    /* Check for a UART receive interrupt condition (UxRIF). */

    if (REG(UARTIFS) & UART_INT_FLAGS(1, UxRIF))
    {
        /* Write the received data back. */

        while (REG(UART_REG(1, UxSTA)) & 1)
            uart_write((char) REG(UART_REG(1, UxRXREG)));

        /* Clear the UART interrupt condition. */

        CLR_REG(UARTIFS, UART_INT_FLAGS(1, UxRIF));
        INV_REG(PORTA, 1 << 2);
    }
}
