/*
 * PIC32 microcontroller initialisation code.
 *
 * Copyright (C) 2017, 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "mips.h"
#include "pic32.h"

/* Disable JTAG functionality on pins. */

.section .devcfg0, "a"
.word 0xfffffffb	/* DEVCFG0<2> = JTAGEN = 0 */

/*
Set the oscillator to be the FRC oscillator with PLL, with peripheral clock
divided by 2, and FRCDIV+PLL selected.

The watchdog timer (FWDTEN) is also disabled.

The secondary oscillator pin (FSOSCEN) is disabled to avoid pin conflicts with
RPB4.
*/

.section .devcfg1, "a"
.word 0xff7fdfd9	/* DEVCFG1<23> = FWDTEN = 0; DEVCFG1<13:12> = FPBDIV<1:0> = 1;
			DEVCFG1<5> = FSOSCEN = 0; DEVCFG1<2:0> = FNOSC<2:0> = 001 */

/*
Set the FRC oscillator PLL function with an input division of 4, an output
division of 2, a multiplication of 24, yielding a multiplication of 3.

The FRC is apparently at 16MHz and this produces a system clock of 48MHz.

The peripheral clock frequency (FPB) will be 24MHz given the above DEVCFG1
settings.
*/

.section .devcfg2, "a"
.word 0xfff9fffb	/* DEVCFG2<18:16> = FPLLODIV<2:0> = 001;
			DEVCFG2<6:4> = FPLLMUL<2:0> = 111;
			DEVCFG2<2:0> = FPLLIDIV<2:0> = 011 */

/* The start routine is placed at the boot location. */

.section .boot, "a"

.globl _start
.extern main

_start:
	/* Enable caching. */

	mfc0 $v1, CP0_CONFIG
	li $t8, ~CONFIG_K0
	and $v1, $v1, $t8
	ori $v1, $v1, CONFIG_K0_CACHABLE_NONCOHERENT
	mtc0 $v1, CP0_CONFIG
	nop

	/* Get the RAM size. */

	la $v1, BMXDRMSZ
	lw $t0, 0($v1)

        /* Initialise the stack pointer. */

        li $v1, KSEG0_BASE
        addu $sp, $t0, $v1      /* sp = KSEG0_BASE + RAM size */

        /* Initialise the globals pointer. */

        lui $gp, %hi(_GLOBAL_OFFSET_TABLE_)
        ori $gp, $gp, %lo(_GLOBAL_OFFSET_TABLE_)

	/*
	Jump to the main program. Since the boot code is separate from the
	other code, the address cannot be obtained via the GOT.
	("relocation truncated to fit: R_MIPS_PC16 against `main'")
	*/

        lui $t9, %hi(main)
        ori $t9, $t9, %lo(main)
	jr $t9
	nop
